@@ -1,19 +1,24 @@
 use std::{
     fs::File,
     io::{Read, Seek, SeekFrom},
-    net::{Ipv4Addr, Ipv6Addr},
-    path::Path,
+    net::{IpAddr, Ipv6Addr},
+    path::{Path, PathBuf},
     result::Result,
-    str::FromStr,
 };
 
 use memmap::Mmap;
 
 use crate::{consts, error, record};
+use crate::error::Error;
+
+#[derive(Debug)]
+struct UnopenedDB {
+    path: PathBuf,
+}
 
 #[derive(Debug)]
 pub struct DB {
-    path: String,
+    path: PathBuf,
     db_type: u8,
     db_column: u8,
     db_year: u8,
@@ -25,19 +30,61 @@ pub struct DB {
     ipv6_db_addr: u32,
     ipv4_index_base_addr: u32,
     ipv6_index_base_addr: u32,
-    file: Option<File>,
-    mmap: Option<Mmap>,
+    source: Source,
 }
 
-impl Drop for DB {
-    fn drop(&mut self) {
-        // The 'file' field will be dropped automatically by Rust.
-        match &mut self.mmap {
-            Some(mmap) => {
-                drop(mmap);
-                self.mmap = None;
+#[derive(Debug)]
+enum Source {
+    File(File),
+    Mmap(Mmap),
+}
+
+impl UnopenedDB {
+    pub fn new<P: AsRef<Path>>(path: P) -> Self {
+        UnopenedDB {
+            path: path.as_ref().to_path_buf(),
+        }
+    }
+
+    /// Consume the unopened db and open the file.
+    fn open(self) -> Result<DB, error::Error> {
+        match File::open(&self.path) {
+            Ok(f) => {
+                let db = DB::new(self, Source::File(f));
+                Ok(db)
+            }
+            Err(e) => {
+                Err(error::Error::IoError(format!(
+                    "Error opening DB file: {}",
+                    e
+                )))
+            }
+        }
+    }
+
+    /// Consume the unopened db and mmap the file.
+    fn mmap(self) -> Result<DB, error::Error> {
+        match File::open(&self.path) {
+            Ok(f) => {
+                match unsafe { Mmap::map(&f) } {
+                    Ok(mmap) => {
+                        let db = DB::new(self, Source::Mmap(mmap));
+                        Ok(db)
+                    }
+                    Err(e) => {
+                        Err(error::Error::IoError(format!(
+                            "error while mmaping db file: {}",
+                            e
+                        )))
+                    }
+                }
+            }
+            Err(e) => {
+                Err(error::Error::IoError(format!(
+                    "Error opening DB file: {}",
+                    e
+                )))
             }
-            None => {}
         }
     }
 }
@@ -53,11 +100,10 @@ impl DB {
         //!
         //! let mut db = DB::from_file("data/IP2LOCATION-LITE-DB1.BIN").unwrap();
         //!```
-        let mut obj = Self::empty();
-        obj.path = path.as_ref().to_string_lossy().into();
-        obj.open()?;
-        obj.read_header()?;
-        Ok(obj)
+        let obj = UnopenedDB::new(path);
+        let mut db = obj.open()?;
+        db.read_header()?;
+        Ok(db)
     }
 
     pub fn from_file_mmap<P: AsRef<Path>>(path: P) -> Result<Self, error::Error> {
@@ -71,11 +117,10 @@ impl DB {
         //!
         //! let mut db = DB::from_file_mmap("data/IP2LOCATION-LITE-DB1.BIN").unwrap();
         //!```
-        let mut obj = Self::empty();
-        obj.path = path.as_ref().to_string_lossy().into();
-        obj.mmap()?;
-        obj.read_header()?;
-        Ok(obj)
+        let obj = UnopenedDB::new(path);
+        let mut db = obj.mmap()?;
+        db.read_header()?;
+        Ok(db)
     }
 
     pub fn print_db_info(&self) {
@@ -89,22 +134,16 @@ impl DB {
         //! let mut db = DB::from_file_mmap("data/IP2LOCATION-LITE-DB1.BIN").unwrap();
         //! db.print_db_info();
         //! ```
-        println!("Db Path: {}", self.path);
+        println!("Db Path: {}", self.path.display());
         println!(" |- Db Type: {}", self.db_type);
         println!(" |- Db Column: {}", self.db_column);
         println!(
             " |- Db Date (YY/MM/DD): {}/{}/{}",
             self.db_year, self.db_month, self.db_day
         );
-        println!(" |- IPv4 Count: {}", self.ipv4_db_count);
-        println!(" |- IPv4 Address: {}", self.ipv4_db_addr);
-        println!(" |- IPv4 Index Base Address: {}", self.ipv4_index_base_addr);
-        println!(" |- IPv6 Count: {}", self.ipv6_db_count);
-        println!(" |- IPv6 Address: {}", self.ipv6_db_addr);
-        println!(" |- IPv6 Index Base Address: {}", self.ipv6_index_base_addr);
     }
 
-    pub fn ip_lookup(&mut self, ip: &str) -> Result<record::Record, error::Error> {
+    pub fn ip_lookup(&mut self, ip: IpAddr) -> Result<record::Record, error::Error> {
         //! Lookup for the given IPv4 or IPv6 and returns the Geo information
         //!
         //! ## Example usage
@@ -113,28 +152,44 @@ impl DB {
         //! use ip2location::DB;
         //!
         //! let mut db = DB::from_file("data/IP2LOCATION-LITE-DB1.IPV6.BIN").unwrap();
-        //! let geo_info = db.ip_lookup("2a01:cb08:8d14::").unwrap();
+        //! let geo_info = db.ip_lookup("2a01:cb08:8d14::".parse().unwrap()).unwrap();
         //! println!("{:#?}", geo_info);
         //! assert!(!geo_info.country.is_none());
         //! assert_eq!(geo_info.country.unwrap().short_name, "FR")
         //!```
-        if Self::is_ipv4(ip) {
-            let ip_number = u32::from(Ipv4Addr::from_str(ip)?);
-            let mut record = self.ipv4_lookup(ip_number)?;
-            record.ip = ip.into();
-            return Ok(record);
-        } else if Self::is_ipv6(ip) {
-            let ipv6 = Ipv6Addr::from_str(ip)?;
-            let mut record = self.ipv6_lookup(ipv6)?;
-            record.ip = ip.into();
-            return Ok(record);
+        match ip {
+            IpAddr::V4(ipv4) => {
+                let mut record = self.ipv4_lookup(u32::from(ipv4))?;
+                record.ip = ip;
+                return Ok(record);
+            }
+            IpAddr::V6(ipv6) => {
+                if let Some(converted_ip) = ipv6.to_ipv4() {
+                    let mut record = self.ipv4_lookup(u32::from(converted_ip))?;
+                    record.ip = ip.into();
+                    return Ok(record);
+                } else if Ipv6Addr::from(consts::FROM_6TO4) <= ipv6 && ipv6 <= Ipv6Addr::from(consts::TO_6TO4) {
+                    let ipnum = (u128::from(ipv6) >> 80) as u32;
+                    let mut record = self.ipv4_lookup(ipnum)?;
+                    record.ip = ip.into();
+                    return Ok(record);
+                } else if Ipv6Addr::from(consts::FROM_TEREDO) <= ipv6 && ipv6 <= Ipv6Addr::from(consts::TO_TEREDO) {
+                    let ipnum = !u128::from(ipv6) as u32;
+                    let mut record = self.ipv4_lookup(ipnum)?;
+                    record.ip = ip.into();
+                    return Ok(record);
+                } else {
+                    let mut record = self.ipv6_lookup(ipv6)?;
+                    record.ip = ip.into();
+                    return Ok(record);
+                }
+            }
         }
-        Err(error::Error::InvalidIP("ip address is invalid".into()))
     }
 
-    fn empty() -> Self {
+    fn new(db: UnopenedDB, source: Source) -> Self {
         Self {
-            path: "".into(),
+            path: db.path,
             db_type: 0,
             db_column: 0,
             db_year: 0,
@@ -146,51 +201,10 @@ impl DB {
             ipv6_db_addr: 0,
             ipv4_index_base_addr: 0,
             ipv6_index_base_addr: 0,
-            file: None,
-            mmap: None,
+            source,
         }
     }
 
-    fn open(&mut self) -> Result<(), error::Error> {
-        match File::open(self.path.clone()) {
-            Ok(f) => {
-                self.file = Some(f);
-                return Ok(());
-            }
-            Err(e) => {
-                return Err(error::Error::IoError(format!(
-                    "Error opening DB file: {}",
-                    e
-                )))
-            }
-        };
-    }
-
-    fn mmap(&mut self) -> Result<(), error::Error> {
-        match File::open(self.path.clone()) {
-            Ok(f) => {
-                match unsafe { Mmap::map(&f) } {
-                    Ok(mmap) => {
-                        self.mmap = Some(mmap);
-                        return Ok(());
-                    }
-                    Err(e) => {
-                        return Err(error::Error::IoError(format!(
-                            "error while mmaping db file: {}",
-                            e
-                        )))
-                    }
-                };
-            }
-            Err(e) => {
-                return Err(error::Error::IoError(format!(
-                    "Error opening DB file: {}",
-                    e
-                )))
-            }
-        };
-    }
-
     fn read_header(&mut self) -> Result<(), error::Error> {
         self.db_type = self.read_u8(1)?;
         self.db_column = self.read_u8(2)?;
@@ -233,7 +247,7 @@ impl DB {
                 }
             }
         }
-        Err("no record found".into())
+        Err(Error::RecordNotFound)
     }
 
     fn ipv6_lookup(&mut self, ipv6: Ipv6Addr) -> Result<record::Record, error::Error> {
@@ -264,11 +278,11 @@ impl DB {
                 }
             }
         }
-        Err("no record found".into())
+        Err(Error::RecordNotFound)
     }
 
     fn read_record(&mut self, row_addr: u32) -> Result<record::Record, error::Error> {
-        let mut result = record::Record::new_empty();
+        let mut result = record::Record::default();
 
         if consts::COUNTRY_POSITION[self.db_type as usize] > 0 {
             let index = self.read_u32(
@@ -406,88 +420,97 @@ impl DB {
             result.usage_type = Some(self.read_str(index.into())?);
         }
 
+        if consts::ADDRESSTYPE_POSITION[self.db_type as usize] > 0 {
+            let index = self.read_u32(
+                (row_addr + 4 * (consts::ADDRESSTYPE_POSITION[self.db_type as usize] - 1)).into(),
+            )?;
+            result.address_type = Some(self.read_str(index.into())?);
+        }
+
+        if consts::CATEGORY_POSITION[self.db_type as usize] > 0 {
+            let index = self.read_u32(
+                (row_addr + 4 * (consts::CATEGORY_POSITION[self.db_type as usize] - 1)).into(),
+            )?;
+            result.category = Some(self.read_str(index.into())?);
+        }
+
         Ok(result)
     }
 
     fn read_u8(&mut self, offset: u64) -> Result<u8, error::Error> {
-        if self.file.is_some() {
-            let mut f = self.file.as_ref().unwrap();
-            f.seek(SeekFrom::Start(offset - 1))?;
-            let mut buf = [0_u8; 1];
-            f.read(&mut buf)?;
-            Ok(buf[0])
-        } else if self.mmap.is_some() {
-            let m = self.mmap.as_ref().unwrap();
-            Ok(m[(offset - 1) as usize])
-        } else {
-            Err(error::Error::InvalidState("db is not open".into()))
+        match &mut self.source {
+            Source::File(f) => {
+                f.seek(SeekFrom::Start(offset - 1))?;
+                let mut buf = [0_u8; 1];
+                f.read(&mut buf)?;
+                Ok(buf[0])
+            }
+            Source::Mmap(m) => {
+                Ok(m[(offset - 1) as usize])
+            }
         }
     }
 
     fn read_u32(&mut self, offset: u64) -> Result<u32, error::Error> {
-        if self.file.is_some() {
-            let mut f = self.file.as_ref().unwrap();
-            f.seek(SeekFrom::Start(offset - 1))?;
-            let mut buf = [0_u8; 4];
-            f.read(&mut buf)?;
-            let result = u32::from_ne_bytes(buf);
-            Ok(result)
-        } else if self.mmap.is_some() {
-            let m = self.mmap.as_ref().unwrap();
-            let mut buf = [0_u8; 4];
-            buf[0] = m[(offset - 1) as usize];
-            buf[1] = m[offset as usize];
-            buf[2] = m[(offset + 1) as usize];
-            buf[3] = m[(offset + 2) as usize];
-            let result = u32::from_ne_bytes(buf);
-            Ok(result)
-        } else {
-            Err(error::Error::InvalidState("db is not open".into()))
+        match &mut self.source {
+            Source::File(f) => {
+                f.seek(SeekFrom::Start(offset - 1))?;
+                let mut buf = [0_u8; 4];
+                f.read(&mut buf)?;
+                let result = u32::from_ne_bytes(buf);
+                Ok(result)
+            }
+            Source::Mmap(m) => {
+                let mut buf = [0_u8; 4];
+                buf[0] = m[(offset - 1) as usize];
+                buf[1] = m[offset as usize];
+                buf[2] = m[(offset + 1) as usize];
+                buf[3] = m[(offset + 2) as usize];
+                let result = u32::from_ne_bytes(buf);
+                Ok(result)
+            }
         }
     }
 
     fn read_f32(&mut self, offset: u64) -> Result<f32, error::Error> {
-        if self.file.is_some() {
-            let mut f = self.file.as_ref().unwrap();
-            f.seek(SeekFrom::Start(offset - 1))?;
-            let mut buf = [0_u8; 4];
-            f.read(&mut buf)?;
-            let result = f32::from_ne_bytes(buf);
-            Ok(result)
-        } else if self.mmap.is_some() {
-            let m = self.mmap.as_ref().unwrap();
-            let mut buf = [0_u8; 4];
-            buf[0] = m[(offset - 1) as usize];
-            buf[1] = m[offset as usize];
-            buf[2] = m[(offset + 1) as usize];
-            buf[3] = m[(offset + 2) as usize];
-            let result = f32::from_ne_bytes(buf);
-            Ok(result)
-        } else {
-            Err(error::Error::InvalidState("db is not open".into()))
+        match &mut self.source {
+            Source::File(f) => {
+                f.seek(SeekFrom::Start(offset - 1))?;
+                let mut buf = [0_u8; 4];
+                f.read(&mut buf)?;
+                let result = f32::from_ne_bytes(buf);
+                Ok(result)
+            }
+            Source::Mmap(m) => {
+                let mut buf = [0_u8; 4];
+                buf[0] = m[(offset - 1) as usize];
+                buf[1] = m[offset as usize];
+                buf[2] = m[(offset + 1) as usize];
+                buf[3] = m[(offset + 2) as usize];
+                let result = f32::from_ne_bytes(buf);
+                Ok(result)
+            }
         }
     }
 
     fn read_str(&mut self, offset: u64) -> Result<String, error::Error> {
-        if self.file.is_some() {
-            let len = self.read_u8(offset + 1)? as usize;
-            let mut f = self.file.as_ref().unwrap();
-            f.seek(SeekFrom::Start(offset + 1))?;
-            let mut buf = vec![0_u8; len];
-            f.read(&mut buf)?;
-            let s = std::str::from_utf8(&buf)?;
-            Ok(s.into())
-        } else if self.mmap.is_some() {
-            let len = self.read_u8(offset + 1)? as usize;
-            let m = self.mmap.as_ref().unwrap();
-            let mut buf = vec![0_u8; len];
-            for i in 0..len {
-                buf[i] = m[(offset + 1) as usize + i];
+        let len = self.read_u8(offset + 1)? as usize;
+        match &mut self.source {
+            Source::File(f) => {
+                f.seek(SeekFrom::Start(offset + 1))?;
+                let mut buf = vec![0_u8; len];
+                f.read(&mut buf)?;
+                let s = String::from_utf8(buf)?;
+                Ok(s)
+            }
+            Source::Mmap(m) => {
+                let mut buf = vec![0_u8; len];
+                for i in 0..len {
+                    buf[i] = m[(offset + 1) as usize + i];
+                }
+                let s = String::from_utf8(buf)?;
+                Ok(s)
             }
-            let s = std::str::from_utf8(&buf)?;
-            Ok(s.into())
-        } else {
-            Err(error::Error::InvalidState("db is not open".into()))
         }
     }
 
@@ -505,26 +528,4 @@ impl DB {
         let result = Ipv6Addr::from(buf);
         Ok(result)
     }
-
-    fn is_ipv4(ip: &str) -> bool {
-        match Ipv4Addr::from_str(ip) {
-            Ok(_) => {
-                true
-            }
-            Err(_) => {
-                false
-            }
-        }
-    }
-
-    fn is_ipv6(ip: &str) -> bool {
-        match Ipv6Addr::from_str(ip) {
-            Ok(_) => {
-                true
-            }
-            Err(_) => {
-                false
-            }
-        }
-    }
 }
