@@ -4,9 +4,7 @@ use std::{fmt, io};
 pub enum Error {
     GenericError(String),
     IoError(String),
-    RecordNotFound(String),
-    InvalidIP(String),
-    InvalidState(String),
+    RecordNotFound,
 }
 
 impl From<io::Error> for Error {
@@ -21,8 +19,8 @@ impl From<&str> for Error {
     }
 }
 
-impl From<std::str::Utf8Error> for Error {
-    fn from(err: std::str::Utf8Error) -> Error {
+impl From<std::string::FromUtf8Error> for Error {
+    fn from(err: std::string::FromUtf8Error) -> Error {
         Error::GenericError(err.to_string())
     }
 }
@@ -44,9 +42,7 @@ impl fmt::Debug for Error {
         match self {
             Error::GenericError(msg) => write!(f, "GenericError: {}", msg)?,
             Error::IoError(msg) => write!(f, "IoError: {}", msg)?,
-            Error::RecordNotFound(msg) => write!(f, "RecordNotFound: {}", msg)?,
-            Error::InvalidIP(msg) => write!(f, "InvalidIP: {}", msg)?,
-            Error::InvalidState(msg) => write!(f, "InvalidState: {}", msg)?,
+            Error::RecordNotFound => write!(f, "RecordNotFound: no record found")?,
         }
         Ok(())
     }
