@@ -18,20 +18,7 @@ fn test_error_display() {
     );
 
     assert_eq!(
-        format!("{:?}", Error::RecordNotFound("no record found".to_string())),
+        format!("{:?}", Error::RecordNotFound),
         "RecordNotFound: no record found".to_string()
     );
-
-    assert_eq!(
-        format!(
-            "{:?}",
-            Error::InvalidIP("ip address is invalid".to_string())
-        ),
-        "InvalidIP: ip address is invalid".to_string()
-    );
-
-    assert_eq!(
-        format!("{:?}", Error::InvalidState("invalid state".to_string())),
-        "InvalidState: invalid state".to_string()
-    );
 }
